==> kslurm-0.4.0rc8-py3-none-any.whl: kslurm/slurm/slurm_command.py <==
from __future__ import absolute_import

import os
import sys
from typing import List, Union

import kslurm.appconfig as appconfig
import kslurm.models.job_templates as templates
import kslurm.slurm.helpers as helpers
from kslurm.args.command import ParsedArgs
from kslurm.exceptions import TemplateError, ValidationError
from kslurm.models.slurm import SlurmModel


class SlurmCommand:
    def __init__(
        self,
        args: Union[SlurmModel, TemplateError],
        tail: list[str],
        arglist: ParsedArgs,
    ):
        self._name = ""
        self._output = ""

        if isinstance(args, TemplateError):
            print(args.msg)
            print("Choose from the following list of templates:\n")
            templates.list_templates()
            exit()

        if args.list_job_templates:
            templates.list_templates()
            exit()

        # set_template returns templated values only if a template is passed
        # if we pass a blank string, the models are returned unchanged
        template = args.job_template[0] if args.job_template else ""
        template_vals = templates.set_template(
            template, mem=args.mem, cpu=args.cpu, time=args.time
        )

        # Start by setting these three with model/template
        self.time = template_vals.time
        self.cpu = template_vals.cpu
        self.mem = template_vals.mem

        # Then update if values were specifically supplied on the command line
        if arglist["time"].updated:
            self.time = args.time
        if arglist["cpu"].updated:
            self.cpu = args.cpu
        if arglist["mem"].updated:
            self.mem = args.mem

        self.gpu = bool(args.gpu)
        self.x11 = bool(args.x11)
        config = appconfig.Config()
        if not args.account:
            if self.gpu:
                account = config.get("account.gpu")
            else:
                account = config.get("account.cpu")
            if account is None:
                account = config.get("account")
            if not account:
                print(
                    "Account must either be specified using --account, or provided in "
                    "config. A default account can be added to config by running "
                    "kslurm config account <account_name>"
                )
                sys.exit(1)
            self.account = account
        else:
            self.account = args.account[0]
        self.cwd = args.directory
        self.test = args.test
        self.job_template = args.job_template
        self._command = tail

        os.chdir(self.cwd)

        self.script = [self.command]

        self.args = args

    ###
    # Job Paramaters
    ###
    @property
    def time(self):
        return helpers.slurm_time_format(self._time)

    @time.setter
    def time(self, time: int):
        self._time = time

    @property
    def name(self):
        if not self._name:
            return self._command[0]
        else:
            return self._name

    @name.setter
    def name(self, name: str):
        self._name = name

    @property
    def output(self):
        return self._output

    @output.setter
    def output(self, output: str):
        self._output = f'--output="{output}"'

    ###
    # Command line strings
    ###
    @property
    def slurm_args(self):
        s = (
            f"--account={self.account} --time={self.time} "
            f"--cpus-per-task={self.cpu} --mem={self.mem}"
        )
        if self.gpu:
            s += " --gres=gpu:1"
        if self.x11:
            s += " --x11"
        return s

    @property
    def command(self):
        return " ".join(self._command)

    @command.setter
    def command(self, commands: List[str]):
        self._command = commands

    @property
    def script(self):
        return self._script

    @script.setter
    def script(self, command: List[str]):
        self._script = "\n".join(["#!/bin/bash"] + command)

    ###
    # Job submission commands
    ###
    @property
    def run(self):
        if self.command:
            return f"echo '{self.command}' | srun {self.slurm_args} bash"
        else:
            return f"salloc {self.slurm_args}"

    @property
    def batch(self):
        if self.test:
            s = "cat"
        else:
            s = (
                f"sbatch {self.slurm_args} --job-name={self.name} "
                f"--parsable {self.output}"
            )
        if self.command:
            return f"echo '{self.script}' | {s}"
        else:
            raise ValidationError("No command given")

==> kslurm-0.5.0-py3-none-any.whl: kslurm/slurm/slurm_command.py <==
from __future__ import absolute_import

import importlib.resources as impr
import os
import shlex
import sys
from pathlib import Path
from typing import List, Union

import kslurm.appconfig as appconfig
import kslurm.models.job_templates as templates
import kslurm.slurm.helpers as helpers
from kslurm.args.command import ParsedArgs
from kslurm.exceptions import TemplateError, ValidationError
from kslurm.models.slurm import SlurmModel


class SlurmCommand:
    def __init__(
        self,
        args: Union[SlurmModel, TemplateError],
        tail: list[str],
        arglist: ParsedArgs,
    ):
        self._name = ""
        self._output = ""

        if isinstance(args, TemplateError):
            print(args.msg)
            print("Choose from the following list of templates:\n")
            templates.list_templates()
            exit()

        if args.list_job_templates:
            templates.list_templates()
            exit()

        # set_template returns templated values only if a template is passed
        # if we pass a blank string, the models are returned unchanged
        template = args.job_template[0] if args.job_template else ""
        template_vals = templates.set_template(
            template, mem=args.mem, cpu=args.cpu, time=args.time
        )

        # Start by setting these three with model/template
        self.time = template_vals.time
        self.cpu = template_vals.cpu
        self.mem = template_vals.mem

        # Then update if values were specifically supplied on the command line
        if arglist["time"].updated:
            self.time = args.time
        if arglist["cpu"].updated:
            self.cpu = args.cpu
        if arglist["mem"].updated:
            self.mem = args.mem

        self.gpu = bool(args.gpu)
        self.x11 = bool(args.x11)
        config = appconfig.Config()
        if not args.account:
            if self.gpu:
                account = config.get("account.gpu")
            else:
                account = config.get("account.cpu")
            if account is None:
                account = config.get("account")
            if not account:
                print(
                    "Account must either be specified using --account, or provided in "
                    "config. A default account can be added to config by running "
                    "kslurm config account <account_name>"
                )
                sys.exit(1)
            self.account = account
        else:
            self.account = args.account[0]
        self.cwd = args.directory
        self.test = args.test
        self.job_template = args.job_template
        self._command = tail

        os.chdir(self.cwd)

        self.args = args
        self._venv = args.venv[0] if args.venv else None

    ###
    # Job Paramaters
    ###
    @property
    def time(self):
        return helpers.slurm_time_format(self._time)

    @time.setter
    def time(self, time: int):
        self._time = time

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name: str):
        self._name = name

    @property
    def output(self):
        return self._output

    @output.setter
    def output(self, output: str):
        self._output = f'--output="{output}"'

    def set_venv(self, name: str):
        self._venv = name

    ###
    # Command line strings
    ###
    @property
    def slurm_args(self):
        s = (
            f"--account={self.account} --time={self.time} "
            f"--cpus-per-task={self.cpu} --mem={self.mem}"
        )
        if self.gpu:
            s += " --gres=gpu:1"
        if self.x11:
            s += " --x11"
        if self.name:
            s += f" --job-name='{self.name}'"
        return s

    @property
    def command(self):
        return " ".join(self._command)

    @command.setter
    def command(self, commands: List[str]):
        self._command = commands

    @property
    def script(self):
        return "\n".join(["#!/bin/bash", self.venv, self.command])

    @property
    def venv(self):
        if self._venv:
            with impr.path("kslurm.bin", "kpy-wrapper.sh") as path:
                return f"source {path}; kpy load {self._venv}; "
        return ""

    ###
    # Job submission commands
    ###
    @property
    def run(self):
        if self.command:
            if os.environ.get("SLURM_JOB_ID") or os.environ.get("SLURM_JOBID"):
                return f"srun {self.slurm_args} {self.command}"
            command = self.venv + self.command
            return f"srun {self.slurm_args} bash -c {shlex.quote(command)}"
        command = (
            "srun --pty bash -c "
            + shlex.quote(f'bash --init-file <(echo "{self.venv}";)')
            if self.venv
            else ""
        )
        return f"salloc {self.slurm_args} {command}"

    @property
    def batch(self):
        if self.test:
            s = "cat"
        else:
            s = f"sbatch {self.slurm_args} --parsable {self.output}"
        if self.command:
            if "/" in self._command[0] and Path(self._command[0]).is_file():
                return f"{s} {self.command}"
            return f"echo {shlex.quote(self.script)} | {s}"
        else:
            raise ValidationError("No command given")

    @property
    def batch_test(self):
        s = f"sbatch {self.slurm_args} --test-only"
        if self.command:
            return f"echo {shlex.quote(self.script)} | {s}"
        else:
            raise ValidationError("No command given")
